const GraphemeSplitter = require('grapheme-splitter');
const splitter = new GraphemeSplitter();
const path = require('path');
const fs = require('fs');

const fontWidthCSVCache = {};
const fontWidthMapCache = {};
const defaultSettings = { font: 'Arial', size: 13 };

function getWidth (string, { font, size, bold, italic }) {
  font = font.toLowerCase() || defaultSettings.font;
  size = size || defaultSettings.size;
  const variant = 0 + (bold ? 1 : 0) + (italic ? 2 : 0);

  const widthsCSV = fontWidthCSVCache[font] || fs.readFileSync(path.resolve(__dirname, `../fonts/${font.toLowerCase()}.csv`), 'utf-8');
  fontWidthCSVCache[font] = widthsCSV;

  const widthsMap = fontWidthMapCache[font] || widthsCSV
    .split('\n')
    .map(
      (codePointLine) => codePointLine
        .split(',')
        .map(codePointElement => parseInt(codePointElement, 10)
      )
    )
    .reduce((widthMap, codePointArr) => {
      const codePoint = codePointArr[0];
      const widthsArr = codePointArr.slice(1);
      widthMap.set(codePoint, widthsArr);
      return widthMap;
    }, new Map());
  fontWidthMapCache[font] = widthsMap;

  let totalWidth = 0;
  splitter.splitGraphemes(string).forEach(function (char) {
    totalWidth += (widthsMap.get(char.codePointAt()) || widthsMap.get(65))[variant];
  });
  return Math.round((totalWidth * (size / 100)) * 100) / 100;
}

module.exports = getWidth;
