/* global describe, it */

const getWidth = require('../src/getWidth');
const assert = require('chai').assert;

describe('getWidth', () => {
  it('should calculate pixel width of simple string', () => {
    assert.equal(getWidth('test string', { font: 'Arial', size: '13' }), 56.55);
    assert.equal(getWidth('test string', { font: 'Arial', size: '16' }), 69.6);
  });

  it('should calculate pixel width of spaces', () => {
    assert.equal(getWidth('test string', { font: 'Arial', size: '13' }), 56.55);
    assert.equal(getWidth('teststring', { font: 'Arial', size: '13' }), 52.91);
  });

  it('should calculate pixel width of latin characters', () => {
    assert.equal(getWidth('#$%&', { font: 'Arial', size: '13' }), 34.84);
    assert.equal(getWidth('¡¢£¤¥¦§¨©ª«¬®¯°±´µ¶·¸»¿', { font: 'Arial', size: '16' }), 183.84);
    assert.equal(getWidth('ÅÆÇËÏÐÑÖ×ØÜÝÞåæçëïðñö÷øüýþÿ', { font: 'Arial', size: '13' }), 217.49);
  });

  it('should calculate pixel width of numbers', () => {
    assert.equal(getWidth('0123456789', { font: 'Arial', size: '13' }), 72.8);
  });

  it('should calculate pixel width of IPA Extensions', () => {
    assert.equal(getWidth('ɵɶɷɸʉʊʋʐʑʒʓʔʕʖʗʘʙʚʛʜʝʞʟʠʡʢʣʤʥʦʧʨʩʪ', { font: 'Arial', size: '13' }), 265.59);
  });

  it('should calculate pixel width of Greek characters', () => {
    assert.equal(getWidth('ΑΒΓΔΕΖΗΘΙΚΛΜΝΞΟΠΡΣΤΥΦΧΨΩ', { font: 'Arial', size: '13' }), 211.77);
  });

  it('should calculate pixel width of Cyrillic characters', () => {
    assert.equal(getWidth('ЉЊЌЏДИМПљњѦѨѪѬѮѲѴѶѸѺѼѾҀ҂҈҉ҊҎҒҔҖҘҠҢҤҦҧҨ', { font: 'Arial', size: '13' }), 385.45);
  });

  it('should calculate pixel width of Armenian characters', () => {
    assert.equal(getWidth('ԱԲԳԴԵԶԷԸԺԻԼԽԾՀՁՂՃՄՅՆՇՈՉՊՋՌՍՎՏՐՑՒՓՔՕՖ', { font: 'Arial', size: '13' }), 298.87);
  });

  it('should calculate pixel width of Hebrew characters', () => {
    assert.equal(getWidth('א‎צ‎פ‎ס‎מ‎', { font: 'Arial', size: '13' }), 35.88);
  });

  it('should calculate pixel width of Arabic characters', () => {
    assert.equal(getWidth('ؤإئبةتثجحخدذرزسشصضطظغؼؽؾؿـفقكلمنهوىي', { font: 'Arial', size: '13' }), 269.62);
  });

  it('should calculate pixel width of Devanagari characters', () => {
    assert.equal(getWidth('इईउऊऋऌऍऎएऑऒओऔकखगघङचछजझञटीुूृॄ', { font: 'Arial', size: '13' }), 218.66);
  });

  it('should calculate pixel width of Bengali characters', () => {
    assert.equal(getWidth('ওঔকখগঘঙচছজঝঞট', { font: 'Arial', size: '13' }), 114.4);
  });

  it('should calculate pixel width of Gurmukhi characters', () => {
    assert.equal(getWidth('ਓਔਕਖਗਘਙਚਛਜਝ', { font: 'Arial', size: '13' }), 104.78);
  });

  it('should calculate pixel width of Gujarati characters', () => {
    assert.equal(getWidth('અઆઇઈઉઊઋઌઍ', { font: 'Arial', size: '13' }), 95.16);
  });

  it('should calculate pixel width of Oriya characters', () => {
    assert.equal(getWidth('ଅଆଇଈଉଊଋଌ', { font: 'Arial', size: '13' }), 81.25);
  });

  it('should calculate pixel width of Malayalam characters', () => {
    assert.equal(getWidth('അആഇഈഉഊഋഌ', { font: 'Arial', size: '13' }), 116.22);
  });

  it('should calculate pixel width of Thai characters', () => {
    assert.equal(getWidth('กขฃคฅฆงจฉช', { font: 'Arial', size: '13' }), 76.18);
  });

  it('should calculate pixel width of Tibetan characters', () => {
    assert.equal(getWidth('ༀ༁༂༃༄༅༆༇༈༉༊', { font: 'Arial', size: '13' }), 112.58);
  });

  it('should calculate pixel width of Myanmar characters', () => {
    assert.equal(getWidth('ကခဂဃငစဆဇဈဉည', { font: 'Arial', size: '13' }), 117.78);
  });

  it('should calculate pixel width of Georgian characters', () => {
    assert.equal(getWidth('ႠႡႢႣႤႥႦႧႨႩႪ', { font: 'Arial', size: '13' }), 98.41);
  });

  it('should calculate pixel width of Hangul Jamo characters', () => {
    assert.equal(getWidth('ᄐᄑᄒᄓᄔᄕᄖᄗᄘᄙᄚᄛᄜᄝᄞ', { font: 'Arial', size: '13' }), 11.18);
  });

  it('should calculate pixel width of Ethiopic characters', () => {
    assert.equal(getWidth('ሔሕሖሗመሙሚማ', { font: 'Arial', size: '13' }), 119.08);
  });

  it('should calculate pixel width of Khmer characters', () => {
    assert.equal(getWidth('ឃងចឆជឈញដ', { font: 'Arial', size: '13' }), 80.6);
  });

  it('should calculate pixel width of Mongolian characters', () => {
    assert.equal(getWidth('ᠠᠡᠢᠣᠤᠥᠦᠧᠨ', { font: 'Arial', size: '13' }), 58.37);
  });

  it('should calculate pixel width of Phonetic Extensions characters', () => {
    assert.equal(getWidth('ᴐᴑᴒᴓᴔᴕ', { font: 'Arial', size: '13' }), 48.49);
  });

  it('should calculate pixel width of Superscripts and Subscripts characters', () => {
    assert.equal(getWidth('⁰ⁱ₂₃', { font: 'Arial', size: '13' }), 19.24);
  });

  it('should calculate pixel width of Currency Symbols characters', () => {
    assert.equal(getWidth('₤₥₦₧₨₩₪₫€', { font: 'Arial', size: '13' }), 93.47);
  });

  it('should calculate pixel width of Letterlike Symbols characters', () => {
    assert.equal(getWidth('ℽℾℿ⅀⅁', { font: 'Arial', size: '13' }), 45.76);
  });

  it('should calculate pixel width of Number Forms characters', () => {
    assert.equal(getWidth('⅐⅒⅔⅖⅙⅛ⅠⅢⅤⅦⅨⅪⅬⅮⅱⅳⅵⅷⅹⅻⅽⅿ', { font: 'Arial', size: '13' }), 238.94);
  });

  it('should calculate pixel width of Miscellaneous Technical characters', () => {
    assert.equal(getWidth('⌚⌛', { font: 'Arial', size: '13' }), 26);
  });

  it('should calculate pixel width of Enclosed Alphanumerics characters', () => {
    assert.equal(getWidth('①②③⒅⒆⒇', { font: 'Arial', size: '13' }), 78);
  });

  it('should calculate pixel width of Miscellaneous Symbols characters', () => {
    assert.equal(getWidth('☔☕☖☗☘', { font: 'Arial', size: '13' }), 65);
  });

  it('should calculate pixel width of Dingbats characters', () => {
    assert.equal(getWidth('✊✋✌✍', { font: 'Arial', size: '13' }), 52);
  });

  it('should calculate pixel width of Braille Patterns characters', () => {
    assert.equal(getWidth('⡖⡗⡘⡙⡚⡛', { font: 'Arial', size: '13' }), 53.04);
  });

  it('should calculate pixel width of Kangxi Radicals characters', () => {
    assert.equal(getWidth('⽁⽂⽃⽄⽅⽆', { font: 'Arial', size: '13' }), 78);
  });

  it('should calculate pixel width of Hiragana characters', () => {
    assert.equal(getWidth('こごさざしじす', { font: 'Arial', size: '13' }), 91);
  });

  it('should calculate pixel width of CJK Unified Ideographs characters', () => {
    assert.equal(getWidth('亰亱亲亳亴亵亶亷亸亹', { font: 'Arial', size: '13' }), 130);
    assert.equal(getWidth('斠斡斢斣斤斥斦斧', { font: 'Arial', size: '16' }), 128);
  });

  it('should calculate pixel width of Hangul Syllables characters', () => {
    assert.equal(getWidth('갱갲갳갴갵', { font: 'Arial', size: '13' }), 55.9);
    assert.equal(getWidth('얲얳어억얶얷언', { font: 'Arial', size: '16' }), 96.32);
  });

  it('should calculate pixel width of Mahjong Tiles characters', () => {
    assert.equal(getWidth('🀀🀁🀂🀃🀄', { font: 'Arial', size: '13' }), 53.56);
  });

  it('should calculate pixel width of Domino Tiles characters', () => {
    assert.equal(getWidth('🁣🁤🁥🁦🁧🁨🁩🁪🁫', { font: 'Arial', size: '13' }), 62.01);
  });

  it('should calculate pixel width of Playing Cards characters', () => {
    assert.equal(getWidth('🂡🂢🂣🂤🂥🂦🂧🂨🂩🂪🂫🂬🂭', { font: 'Arial', size: '13' }), 118.3);
  });

  it('should calculate pixel width of Miscellaneous Symbols and Pictographs characters', () => {
    assert.equal(getWidth('🎀🎁🎂🎃🎄🎅', { font: 'Arial', size: '13' }), 78);
    assert.equal(getWidth('👆👇👈👉👊👋👌👍👎👏', { font: 'Arial', size: '13' }), 130);
    assert.equal(getWidth('👦👧👨👩', { font: 'Arial', size: '16' }), 64);
    assert.equal(getWidth('👪', { font: 'Arial', size: '16' }), 16);
  });

  it('should calculate pixel width of Emoticons characters', () => {
    assert.equal(getWidth('🙄🙅🙆🙇🙈🙉🙊🙋🙌', { font: 'Arial', size: '13' }), 117);
    assert.equal(getWidth('😀😁😂😃😄😅😆😇😈', { font: 'Arial', size: '16' }), 144);
    assert.equal(getWidth('😥😦😧😨😩😪😫', { font: 'Arial', size: '13' }), 91);
  });

  it('should calculate pixel width of Transport and Map Symbols characters', () => {
    assert.equal(getWidth('🚀🚁🚂🚃🚄🚅', { font: 'Arial', size: '13' }), 78);
    assert.equal(getWidth('🚲🚳🚴🚵🚶🚷', { font: 'Arial', size: '16' }), 96);
    assert.equal(getWidth('🚐🚑🚒🚓🚔🚕', { font: 'Arial', size: '13' }), 78);
  });

  it('should calculate pixel width of Supplemental Symbols and Pictographs characters', () => {
    assert.equal(getWidth('🤠🤡🤢🤣🤤🤥🤦', { font: 'Arial', size: '16' }), 112);
    assert.equal(getWidth('🤐🤑🤒🤓🤔🤕🤖🤗', { font: 'Arial', size: '13' }), 104);
  });

  it('should calculate pixel width of non printable characters', () => {
    assert.equal(getWidth('𖩀', { font: 'Arial', size: '13' }), 9.36);
    assert.equal(getWidth('𖩀', { font: 'Arial', size: '16' }), 11.52);
    assert.equal(getWidth(`${String.fromCodePoint(0x16A44)}`, { font: 'Arial', size: '16' }), 11.52);
  });

  it('should calculate pixel width of for bold and italic texts', () => {
    assert.equal(getWidth('test string', { font: 'Arial', size: '13', bold: true }), 62.92);
    assert.equal(getWidth('test string', { font: 'Arial', size: '13', italic: true }), 56.55);
    assert.equal(getWidth('test string', { font: 'Arial', size: '16', bold: true, italic: true }), 77.44);
  });

  it('should calculate pixel width of long texts', () => {
    assert.equal(getWidth('Start your FREE 14-day trial today. WooRank will help you to address SEO issues on your site. identify opportunities with instant reviews and check your SEO.', { font: 'Arial', size: '13' }), 915.85);
  });

  it('should fail when calculating pixel width for emoticons with skin color', () => {
    assert.notEqual(getWidth('👩🏿‍👩🏿‍👧🏿‍👧🏿', { font: 'Arial', size: '13' }), 52);
  });

  it('should fallback for characters not in width map', () => {
    assert.equal(getWidth('h򬟤llo w򬟤rld', { font: 'Arial', size: '13' }), 65.13);
  });
});
